@@ -12,7 +12,7 @@ import numpy as np
 from scipy.ndimage import median_filter, uniform_filter1d
 
 from ._algorithm_setup import _get_vander, _setup_window
-from .utils import gaussian, gaussian_kernel, padded_convolve
+from .utils import ParameterWarning, gaussian, gaussian_kernel, padded_convolve
 
 
 def noise_median(data, half_window, smooth_half_window=None, sigma=None, **pad_kwargs):
@@ -80,15 +80,15 @@ def snip(data, max_half_window, decreasing=False, smooth_half_window=None,
         The y-values of the measured data, with N data points.
     max_half_window : int or Sequence(int, int)
         The maximum number of iterations. Should be set such that
-        `max_half_window`=(w-1)/2, where w is the index-based width of a
-        feature or peak. `max_half_window` can also be a sequence of two
-        integers for asymmetric peaks, with the first item corresponding to
+        `max_half_window` is approxiamtely ``(w-1)/2``, where ``w`` is the index-based
+        width of a feature or peak. `max_half_window` can also be a sequence of
+        two integers for asymmetric peaks, with the first item corresponding to
         the `max_half_window` of the peak's left edge, and the second item
         for the peak's right edge [3]_.
     decreasing : bool, optional
         If False (default), will iterate through window sizes from 1 to
-        max_half_window. If True, will reverse the order and iterate from
-        max_half_window to 1, which gives a smoother baseline according to [3]_
+        `max_half_window`. If True, will reverse the order and iterate from
+        `max_half_window` to 1, which gives a smoother baseline according to [3]_
         and [4]_.
     smooth_half_window : int, optional
         The half window to use for smoothing the data. If `smooth_half_window`
@@ -128,11 +128,11 @@ def snip(data, max_half_window, decreasing=False, smooth_half_window=None,
 
     If data covers several orders of magnitude, better results can be obtained
     by first transforming the data using log-log-square transform before
-    using SNIP [2]_:
+    using SNIP [2]_::
 
         transformed_data =  np.log(np.log(np.sqrt(data + 1) + 1) + 1)
 
-    and then baseline can then be reverted back to the original scale using inverse:
+    and then baseline can then be reverted back to the original scale using inverse::
 
         baseline = -1 + (np.exp(np.exp(snip(transformed_data)) - 1) - 1)**2
 
@@ -151,7 +151,7 @@ def snip(data, max_half_window, decreasing=False, smooth_half_window=None,
            Physics Research A, 2009, 60, 478-487.
 
     """
-    #TODO potentially add adaptive window sizes from [4]_, or at least allow inputting
+    # TODO potentially add adaptive window sizes from [4]_, or at least allow inputting
     # an array of max_half_windows; would need to have a separate function for array
     # windows since it would no longer be able to be vectorized
     if filter_order not in {2, 4, 6, 8}:
@@ -168,7 +168,8 @@ def snip(data, max_half_window, decreasing=False, smooth_half_window=None,
     for i, half_window in enumerate(half_windows):
         if half_window > (num_y - 1) // 2:
             warnings.warn(
-                'max_half_window values greater than (len(data) - 1) / 2 have no effect.'
+                'max_half_window values greater than (len(data) - 1) / 2 have no effect.',
+                ParameterWarning
             )
             half_windows[i] = (num_y - 1) // 2
 
